import datetime as dt

import pandas_datareader.tsp as tsp


class TestTSPFunds(object):
    def test_get_allfunds(self):
        tspdata = tsp.TSPReader(start='2015-11-2', end='2015-11-2').read()

        assert len(tspdata == 1)

        assert round(tspdata['I Fund'][dt.date(2015, 11, 2)], 5) == 25.0058

    def test_sanitize_response(self):
        class response(object):
            pass
        r = response()
        r.text = ' , '
        ret = tsp.TSPReader._sanitize_response(r)
        assert ret == ''
        r.text = ' a,b '
        ret = tsp.TSPReader._sanitize_response(r)
        assert ret == 'a,b'



import pytest

from requests.exceptions import HTTPError
import pandas_datareader.data as web


class TestMoex(object):
    def test_moex_datareader(self):
        try:
            df = web.DataReader("USD000UTSTOM",
                                "moex",
                                start="2017-07-01",
                                end="2017-07-31")
            assert 'SECID' in df.columns
        except HTTPError as e:
            pytest.skip(e)



# pylint: disable-msg=E1101,W0613,W0603

import os

import numpy as np
import pandas as pd
import pandas.util.testing as tm

from pandas_datareader.io.sdmx import read_sdmx, _read_sdmx_dsd


class TestSDMX(object):

    def setup_method(self, method):
        self.dirpath = tm.get_data_path()

    def test_tourism(self):
        # Eurostat
        # Employed doctorate holders in non managerial and non professional
        # occupations by fields of science (%)
        dsd = _read_sdmx_dsd(os.path.join(self.dirpath, 'sdmx',
                                          'DSD_cdh_e_fos.xml'))
        df = read_sdmx(os.path.join(self.dirpath, 'sdmx',
                                    'cdh_e_fos.xml'), dsd=dsd)

        assert isinstance(df, pd.DataFrame)
        assert df.shape == (2, 336)

        df = df['Percentage']['Total']['Natural sciences']
        df = df[['Norway', 'Poland', 'Portugal', 'Russia']]

        exp_col = pd.MultiIndex.from_product([['Norway', 'Poland', 'Portugal',
                                               'Russia'], ['Annual']],
                                             names=['GEO', 'FREQ'])
        exp_idx = pd.DatetimeIndex(['2009', '2006'], name='TIME_PERIOD')

        values = np.array([[20.38, 25.1, 27.77, 38.1],
                           [25.49, np.nan, 39.05, np.nan]])
        expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
        tm.assert_frame_equal(df, expected)



from pandas import (concat, DataFrame)
from pandas_datareader.yahoo.daily import YahooDailyReader


class YahooActionReader(YahooDailyReader):
    """
    Returns DataFrame of historical corporate actions (dividends and stock
    splits) from symbols, over date range, start to end. All dates in the
    resulting DataFrame correspond with dividend and stock split ex-dates.
    """
    def read(self):
        dividends = YahooDivReader(symbols=self.symbols,
                                   start=self.start,
                                   end=self.end,
                                   retry_count=self.retry_count,
                                   pause=self.pause,
                                   session=self.session).read()
        # Add a label column so we can combine our two DFs
        if isinstance(dividends, DataFrame):
            dividends["action"] = "DIVIDEND"
            dividends = dividends.rename(columns={'Dividends': 'value'})

        splits = YahooSplitReader(symbols=self.symbols,
                                  start=self.start,
                                  end=self.end,
                                  retry_count=self.retry_count,
                                  pause=self.pause,
                                  session=self.session).read()
        # Add a label column so we can combine our two DFs
        if isinstance(splits, DataFrame):
            splits["action"] = "SPLIT"
            splits = splits.rename(columns={'Stock Splits': 'value'})
            # Converts fractional form splits (i.e. "2/1") into conversion
            # ratios, then take the reciprocal
            splits['value'] = splits.apply(lambda x: 1/eval(x['value']), axis=1)  # noqa

        output = concat([dividends, splits]).sort_index(ascending=False)

        return output


class YahooDivReader(YahooDailyReader):

    @property
    def service(self):
        return 'div'


class YahooSplitReader(YahooDailyReader):

    @property
    def service(self):
        return 'split'



import pytest

import pandas as pd
import pandas.util.testing as tm

import pandas_datareader.data as web
from pandas_datareader.famafrench import get_available_datasets


class TestFamaFrench(object):

    def test_get_data(self):
        keys = [
            'F-F_Research_Data_Factors', 'F-F_ST_Reversal_Factor',
            '6_Portfolios_2x3', 'Portfolios_Formed_on_ME',
            'Prior_2-12_Breakpoints', 'ME_Breakpoints',
        ]

        for name in keys:
            ff = web.DataReader(name, 'famafrench')
            assert 'DESCR' in ff
            assert len(ff) > 1

    def test_get_available_datasets(self):
        pytest.importorskip("lxml")
        avail = get_available_datasets()
        assert len(avail) > 100

    def test_index(self):
        ff = web.DataReader('F-F_Research_Data_Factors', 'famafrench')
        assert ff[0].index.freq == 'M'
        assert ff[1].index.freq == 'A-DEC'

    def test_f_f_research(self):
        results = web.DataReader("F-F_Research_Data_Factors", "famafrench",
                                 start='2010-01-01', end='2010-12-01')
        assert isinstance(results, dict)
        assert len(results) == 3

        exp = pd.DataFrame({'Mkt-RF': [-3.36, 3.4, 6.31, 2., -7.89, -5.56,
                                       6.93, -4.77, 9.54, 3.88, 0.6, 6.82],
                            'SMB': [0.38, 1.2, 1.42, 4.98, 0.05, -1.97, 0.16,
                                    -3.00, 3.92, 1.15, 3.70, 0.7],
                            'HML': [0.31, 3.16, 2.1, 2.81, -2.38, -4.5, -0.27,
                                    -1.95, -3.12, -2.59, -0.9, 3.81],
                            'RF': [0., 0., 0.01, 0.01, 0.01, 0.01, 0.01,
                                   0.01, 0.01, 0.01, 0.01, 0.01]},
                           index=pd.period_range('2010-01-01', '2010-12-01',
                                                 freq='M', name='Date'),
                           columns=['Mkt-RF', 'SMB', 'HML', 'RF'])
        tm.assert_frame_equal(results[0], exp)

    def test_me_breakpoints(self):
        results = web.DataReader("ME_Breakpoints", "famafrench",
                                 start='2010-01-01', end='2010-12-01')
        assert isinstance(results, dict)
        assert len(results) == 2
        assert results[0].shape == (12, 21)

        exp_columns = pd.Index(['Count', (0, 5), (5, 10), (10, 15), (15, 20),
                                (20, 25), (25, 30), (30, 35), (35, 40),
                                (40, 45), (45, 50), (50, 55), (55, 60),
                                (60, 65), (65, 70), (70, 75), (75, 80),
                                (80, 85), (85, 90), (90, 95), (95, 100)],
                               dtype='object')
        tm.assert_index_equal(results[0].columns, exp_columns)

        exp_index = pd.period_range('2010-01-01', '2010-12-01',
                                    freq='M', name='Date')
        tm.assert_index_equal(results[0].index, exp_index)

    def test_prior_2_12_breakpoints(self):
        results = web.DataReader("Prior_2-12_Breakpoints", "famafrench",
                                 start='2010-01-01', end='2010-12-01')
        assert isinstance(results, dict)
        assert len(results) == 2
        assert results[0].shape == (12, 22)

        exp_columns = pd.Index(['<=0', '>0', (0, 5), (5, 10), (10, 15),
                                (15, 20), (20, 25), (25, 30), (30, 35),
                                (35, 40), (40, 45), (45, 50), (50, 55),
                                (55, 60), (60, 65), (65, 70), (70, 75),
                                (75, 80), (80, 85), (85, 90), (90, 95),
                                (95, 100)], dtype='object')
        tm.assert_index_equal(results[0].columns, exp_columns)

        exp_index = pd.period_range('2010-01-01', '2010-12-01',
                                    freq='M', name='Date')
        tm.assert_index_equal(results[0].index, exp_index)



from pandas_datareader.base import _DailyBaseReader
from pandas_datareader.exceptions import UnstableAPIWarning

UNSTABLE_WARNING = """
The Google Finance API has not been stable since late 2017. Requests seem
to fail at random. Failure is especially common when bulk downloading.
"""


class GoogleDailyReader(_DailyBaseReader):
    """
    Returns DataFrame/Panel of historical stock prices from symbols, over date
    range, start to end. To avoid being penalized by Google Finance servers,
    pauses between downloading 'chunks' of symbols can be specified.

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    start : string, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : string, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        Number of symbols to download consecutively before intiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used
    """

    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.001, session=None, chunksize=25):
        import warnings
        warnings.warn(UNSTABLE_WARNING, UnstableAPIWarning)
        super(GoogleDailyReader, self).__init__(symbols, start, end,
                                                retry_count, pause, session,
                                                chunksize)

    @property
    def url(self):
        """API URL"""
        return 'https://finance.google.com/finance/historical'

    def _get_params(self, symbol):
        params = {
            'q': symbol,
            'startdate': self.start.strftime('%b %d, %Y'),
            'enddate': self.end.strftime('%b %d, %Y'),
            'output': "csv"
        }
        return params



import os
import pytest

from requests.exceptions import HTTPError
import pandas_datareader as pdr
import pandas_datareader.data as web

TEST_API_KEY = os.getenv('ENIGMA_API_KEY')


@pytest.mark.skipif(TEST_API_KEY is None, reason="no enigma_api_key")
class TestEnigma(object):

    @property
    def dataset_id(self):
        """
        USDA Food Recall Archive - 1996
        Selected for being a relatively small dataset.
        https://public.enigma.com/datasets/292129b0-1275-44c8-a6a3-2a0881f24fe1
        """
        return "292129b0-1275-44c8-a6a3-2a0881f24fe1"

    @classmethod
    def setup_class(cls):
        pytest.importorskip("lxml")

    def test_enigma_datareader(self):
        try:
            df = web.DataReader(self.dataset_id,
                                'enigma', access_key=TEST_API_KEY)
            assert 'case_number' in df.columns
        except HTTPError as e:
            pytest.skip(e)

    def test_enigma_get_data_enigma(self):
        try:
            df = pdr.get_data_enigma(self.dataset_id, TEST_API_KEY)
            assert 'case_number' in df.columns
        except HTTPError as e:
            pytest.skip(e)

    def test_bad_key(self):
        with pytest.raises(HTTPError):
            web.DataReader(self.dataset_id,
                           'enigma', access_key=TEST_API_KEY + 'xxx')

    def test_bad_dataset_id(self):
        with pytest.raises(HTTPError):
            web.DataReader('zzzzzzzz-zzzz-zzzz-zzzz-zzzzzzzzzzz',
                           'enigma', access_key=TEST_API_KEY)



import datetime as dt

import requests
from pandas import to_datetime
from pandas_datareader.compat import is_number
from requests_file import FileAdapter
from requests_ftp import FTPAdapter


class SymbolWarning(UserWarning):
    pass


class RemoteDataError(IOError):
    pass


def _sanitize_dates(start, end):
    """
    Return (datetime_start, datetime_end) tuple
    if start is None - default is 2010/01/01
    if end is None - default is today
    """
    if is_number(start):
        # regard int as year
        start = dt.datetime(start, 1, 1)
    start = to_datetime(start)

    if is_number(end):
        end = dt.datetime(end, 1, 1)
    end = to_datetime(end)

    if start is None:
        start = dt.datetime(2010, 1, 1)
    if end is None:
        end = dt.datetime.today()
    if start > end:
        raise ValueError('start must be an earlier date than end')
    return start, end


def _init_session(session, retry_count=3):
    if session is None:
        session = requests.Session()
        session.mount('file://', FileAdapter())
        session.mount('ftp://', FTPAdapter())
        # do not set requests max_retries here to support arbitrary pause
    return session
